#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <phidget22.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>

#define ON 		1
#define OFF		0
#define MATCH 		0
#define PICTURE_TRUE	1
#define PICTURE_FALSE	0

//You will have to change these serial numbers for your own program
#define DIGITALINPUT_SERIALNUM 		370465 
#define DIGITALOUTPUT_SERIALNUM 	370465
#define DIGITALINPUT_HUBPORT	 	1
#define DIGITALOUTPUT_HUBPORT 		0
#define RFID_SERIALNUM 			377511
#define DICTIONARY_SERIALNUMBER		12345

PhidgetDictionaryHandle dict;
PhidgetDigitalOutputHandle digout;
PhidgetDigitalInputHandle digin;
PhidgetRFIDHandle rfid;


static int datalogfd;
//Make sure this path leads to doorstate.txt on your SBC
#define DATALOG		"/usr/userapps/doorlock/data/doorstate.txt"

static void CCONV ssleep(int);

void
onAttachHandler(PhidgetHandle phid, void *ctx) {
}

void
onDetachHandler(PhidgetHandle phid, void *ctx) {
}

void
errorHandler(PhidgetHandle phid, void *ctx, Phidget_ErrorEventCode errorCode, const char *errorString) {
}

int tagRead = 0;
char name[10];
void CCONV
onTagHandler(PhidgetRFIDHandle ch, void *ctx, const char* tag, PhidgetRFID_Protocol protocol) {
	tagRead = 1;
	strcpy(name, tag); //store tag for use in main loop
	PhidgetLog_log(PHIDGET_LOG_INFO, "got tag - %s", name); //log who unlocked the door
}

int dictionaryUpdate = 0;
void CCONV
onDictionaryUpdate(PhidgetDictionaryHandle ch, void *ctx, const char *key, const char* value){
	if(strcmp(key,"remoteopen") == MATCH) //if someone remotely unlocked the door, set a flag and deal with it in main loop
		if(strcmp(value, "1") == MATCH)
			dictionaryUpdate = 1;
}

void CCONV
onStateChangeHandler(PhidgetDigitalInputHandle ch, void* context, int state){
	char buf[128];
	size_t n;

	PhidgetLog_log(PHIDGET_LOG_INFO, "door state changed: %d", state);

	if(state) //update dictionary so webpage knows what is happening
		PhidgetDictionary_set(dict,"isDoorOpen","Door is closed");
	else
		PhidgetDictionary_set(dict,"isDoorOpen","Door is open");
	
	n = mos_snprintf(buf, sizeof (buf), "%T,%d\n", NULL, state); //write data to doorstate.txt
	write(datalogfd, buf, n);
}

void changeDoorState(PhidgetDigitalOutputHandle ch, int state){
	PhidgetReturnCode result;
	result = PhidgetDigitalOutput_setState(ch, state);
	if(result != EPHIDGET_OK){
		const char* error;
		Phidget_getErrorDescription(result, &error);
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set door state - %s", error);
		return;
	}
	PhidgetLog_log(PHIDGET_LOG_INFO, "door state set to %d", state);
}


int
main(int argc, char **argv) {	PhidgetReturnCode result;
	PhidgetLog_enable(PHIDGET_LOG_INFO, "doorlock.log"); //logfile will store info about when door state has changed as well as general debug info
	PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICE);
	datalogfd = open(DATALOG, O_WRONLY | O_CREAT | O_APPEND, 0664);
	if (datalogfd < 0) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "Failed to open data log file, exiting program");
		return 1;
	}

	//RFID code
	result = PhidgetRFID_create(&rfid);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to create rfid");
		return 1;
	}
	result = Phidget_setDeviceSerialNumber((PhidgetHandle)rfid, RFID_SERIALNUM);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set device serial number");
		return 1;
	}
	result = PhidgetRFID_setOnTagHandler(rfid, onTagHandler, NULL);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set on tag handler");
		return 1;
	}
	result = Phidget_openWaitForAttachment((PhidgetHandle)rfid, 2000);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to open rfid");
		return 1;
	}
	result = PhidgetRFID_setAntennaEnabled(rfid, 1);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set antenna enabled");
		return 1;
	}

	//Digital output code (used to control relay which provides power to solenoid)
	result = PhidgetDigitalOutput_create(&digout);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to create digital output");
		return 1;
	}
	result = Phidget_setDeviceSerialNumber((PhidgetHandle)digout, DIGITALOUTPUT_SERIALNUM);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set device serial number");
		return 1;
	}
	result = Phidget_setHubPort((PhidgetHandle)digout, DIGITALOUTPUT_HUBPORT);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set hub port");
		return 1;
	}
	result = Phidget_setIsHubPortDevice((PhidgetHandle)digout, 1);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set is hub port device");
		return 1;
	}
	result = Phidget_openWaitForAttachment((PhidgetHandle)digout, 2000);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to open digout");
		return 1;
	}

	//Digital input code (used with magenetic contact switch, lets us know real state of door)
	result = PhidgetDigitalInput_create(&digin);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to create digital input");
		return 1;
	}
	result = Phidget_setDeviceSerialNumber((PhidgetHandle)digin, DIGITALINPUT_SERIALNUM);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set device serial number");
		return 1;
	}
	result = Phidget_setHubPort((PhidgetHandle)digin, DIGITALINPUT_HUBPORT);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set hub port");
		return 1;
	}
	result = Phidget_setIsHubPortDevice((PhidgetHandle)digin, 1);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set is hub port device");
		return 1;
	}
	result = PhidgetDigitalInput_setOnStateChangeHandler(digin, onStateChangeHandler, NULL);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set on state change handler");
		return 1;
	}
	result = Phidget_openWaitForAttachment((PhidgetHandle)digin, 2000);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to open digin");
		return 1;
	}

	//Dictionary code (allows webpage code and this program to interact with eachother)
	result = PhidgetDictionary_create(&dict);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to create dictionary");
		return 1;
	}
	result = Phidget_setDeviceSerialNumber((PhidgetHandle)dict, DICTIONARY_SERIALNUMBER);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set serial number");
		return 1;
	}
	result = PhidgetDictionary_setOnUpdateHandler(dict, onDictionaryUpdate, NULL);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to set onUpdateHandler");
		return 1;
	}
	result = Phidget_openWaitForAttachment((PhidgetHandle)dict, 2000);
	if (result != EPHIDGET_OK) {
		PhidgetLog_log(PHIDGET_LOG_ERROR, "failed to open dictionary");
		return 1;
	}


	mainloop:
	while (1) {
		if(dictionaryUpdate){
			PhidgetLog_log(PHIDGET_LOG_INFO,"Remotely opening door");
			dictionaryUpdate = 0;
			changeDoorState(digout, ON);
			ssleep(2);
			changeDoorState(digout,OFF);
			PhidgetDictionary_set(dict,"remoteopen","0");
		}
		if (tagRead) {
			tagRead = 0;
			if (strcmp(name, "Lucas") == MATCH) { //can add multiple RFID tags here for differnet users to access 
				changeDoorState(digout, ON);
				ssleep(2);
				changeDoorState(digout,OFF);
			}
		}
	   sleep(1);
	}

}

static void CCONV
ssleep(int tm) {
#ifdef _WIN32
	Sleep(tm * 1000);
#else
	sleep(tm);
#endif
}